﻿using System;
using System.Windows.Forms;

namespace NodeSelect
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// den TreeView vollständig auslesen
        /// </summary>
        private void btnDump_Click(object sender, EventArgs e)
        {
            // jeden Hauptnode rekursiv auslesen
            foreach (TreeNode n in this.treeView1.Nodes)
            {
                this.DumpRecursive(n);
            }
        }

        /// <summary>
        /// den übergebenen Node rekursiv auslesen
        /// </summary>
        /// <param name="node">der zu dumpende Node</param>
        private void DumpRecursive(TreeNode node)
        {
            // Text des übergebenen Nodes ausgeben
            this.richTextBox1.AppendText(node.Text + "\n");
            // Text jedes Child-Nodes rekursiv ausgeben
            foreach (TreeNode tn in node.Nodes)
            {

                this.DumpRecursive(tn);
            }
        }

        /// <summary>
        /// keinen Node selektieren
        /// </summary>
        private void btnUnselect_Click(object sender, EventArgs e)
        {
            this.treeView1.SelectedNode = null;
        }

        /// <summary>
        /// alle Nodes ausklappen
        /// </summary>
        private void btnExpandAll_Click(object sender, EventArgs e)
        {
            this.treeView1.ExpandAll();
        }

        /// <summary>
        /// alle Nodes einklappen
        /// </summary>
        private void btnCollapseAll_Click(object sender, EventArgs e)
        {
            this.treeView1.CollapseAll();
        }

        /// <summary>
        /// Den Vorgänger-Node selektieren
        /// </summary>
        private void btnPrevious_Click(object sender, EventArgs e)
        {
            TreeNode node = null;
            if (this.treeView1.SelectedNode == null)
            {
                // falls kein Node selektiert ist, wird der letzte Haupt-Node genommen => Anzeige
                // (dies muss nicht der letzt Node überhaupt sein)
                node = this.treeView1.Nodes[this.treeView1.Nodes.Count - 1];
            }
            else
            {
                // Node 9 => Node 8
                // Es wird der vorige Node derselben Ebene vorgeschlagen
                node = this.treeView1.SelectedNode.PrevNode;

                // dessen Parentnode bereitstellen
                TreeNode node2 = this.treeView1.SelectedNode.Parent;

                // wenn er vorhanden ist und keine Sub-Nodes hat => Anzeige
                if ((node != null) &&
                    (node.Nodes != null) &&
                    (node.Nodes.Count > 0))
                {
                    // Node 11 => Node 10
                    // iterativ wird der letzte Subnode des letzten Subnodes gefunden => Anzeige
                    while ((node.Nodes != null) &&
                        (node.Nodes.Count > 0))
                    {
                        node = node.Nodes[node.Nodes.Count - 1];
                    }
                }
                else if ((node == null) &&
                        (node2 != null))
                {
                    // Node 12 => Node 11
                    // es wird in die Parent-Ebene gewechselt => Anzeige
                    node = node2;
                }
            }

            // Node anzeigen und ausklappen
            this.SelectNode(node);
        }

        /// <summary>
        /// Den Nachfolger-Node selektieren
        /// </summary>
        private void btnNext_Click(object sender, EventArgs e)
        {
            TreeNode node = null;
            if (this.treeView1.SelectedNode == null)
            {
                // falls kein Node selektiert ist, wird der erste genommen
                node = this.treeView1.Nodes[0];  // => Anzeige
            }
            else
            {
                node = this.treeView1.SelectedNode;
                if ((node.Nodes != null) &&
                    (node.Nodes.Count > 0))
                {
                    // der erste Node der nächsten Ebene
                    // Node 1 => Node 2
                    node = node.Nodes[0];   // => Anzeige
                }
                else
                {
                    // Node 2 => Node 3
                    // Es wird der nächste Node derselben Ebene vorgeschlagen
                    node = this.treeView1.SelectedNode.NextNode;

                    // dessen Parentnode bereitstellen
                    TreeNode node2 = this.treeView1.SelectedNode.Parent;

                    // letzter Node der aktuellen Ebene
                    while ((node == null) &&
                        (node2 != null))
                    {
                        if (node2 != null)
                        {
                            // Es wird der nächste Node der nächst höheren Ebene vorgeschlagen
                            node = node2.NextNode;
                            if (node == null)
                            {
                                // Node 10 => Node 9 => Node 7
                                // das Parent wechselt eine Ebene nach oben
                                node2 = node2.Parent;
                            }
                        }

                        if (node != null)
                        {
                            // Node 3 => Node 4
                            // es wird in die Parent-Ebene gewechselt => Anzeige
                            node2 = node.Parent;
                        }
                    }
                }
            }

            // Node anzeigen und ausklappen
            this.SelectNode(node);
        }

        /// <summary>
        /// Anzeige und Ausklappen des übergebenen Nodes
        /// bzw. Ende erreicht
        /// </summary>
        /// <param name="node"></param>
        private void SelectNode(TreeNode node)
        {
            if (node == null)
            {
                // Ende erreicht
                MessageBox.Show("Durch");
                return;
            }

            if (this.cbCollapse.Checked)
            {
                // ggf. den Treeview zusammenklappen
                this.treeView1.CollapseAll();
            }

            // Node selektieren
            this.treeView1.SelectedNode = node;
            if (this.treeView1.SelectedNode.Parent != null)
            {
                // den Parent-Node ausklappen
                this.treeView1.SelectedNode.Parent.Expand();
            }
        }
    }
}
